# quakerheritage/__init__.py
"""Collects text data from online pdfs held by the Quaker Meeting House Heritage Project.

Modules exported by this package:

`callFunctions`: Unifies the functional modules and runs the main code to create a formatted Pandas DataFrame from pdfs held by Britain Yearly Meeting\n

`getWebData`: Main web function that collects url data and extracts text from each pdf examined\n

`cleanseData`: Runs Pandas functionality to merge text data into workable DataFrame and creates consistent data quality\n

"""
__version__ = "0.0.3"


# quakerheritage/getWebData.py

"""Main web function that collects url data and extracts text from each pdf examined.

This module allows the user to extract core data from formatted pdfs on the Britain Yearly Meeting website.

This module contains the following functions:

- `getUrls(url)` - Filters the Quaker Heritage Project's website for the url links to pdfs.
- `pdfDataExtract(url)` - Extracts core data text from a single pdf passed in as a url link.
"""

import io
import urllib3
import re

from bs4 import BeautifulSoup, SoupStrainer
import pdfplumber
import requests



def getUrls(url: str) -> list:
    """Short function to isolate links from the chosen web page. As the webpage is pre-selected, they are all known to be pdf files.

    Args:
        url (string): the pre-selected web address.

    Return:
        pdfList(array): a Python list containing all the links extracted from the page. 
    """
    pdfList = []
    response = requests.get(url)
    for link in BeautifulSoup(response.text, 'html.parser', parse_only=SoupStrainer('a')):
        if link.has_attr('href') and link['href'][0] != "#":
            pdfList.append(url + link['href'])
    return pdfList

def pdfDataExtract(url: str) -> dict:
    """Extracts text data from pdfs and passes it into a dictionary

    Args:
        url (string): a single url which must be a pdf file storage location.

    Return:
        itemDict (array): Python dictionary containing keys and values extracted from text.
    """
    itemDict = {}

    http = urllib3.PoolManager()
    temp = io.BytesIO()
    temp.write(http.request("GET", url).data)
    try:    
        pdf = pdfplumber.open(temp)
        all_text = ''
        itemDict["Meeting Name"] = url.split('/')[-1].split('.')[0].replace("%20", " ")
        header_list= pdf.pages[0].extract_text().splitlines()[0:5]
        header_data = [x.strip() for x in header_list if x.strip()]
        itemDict["Meeting Full Name"] = header_data[0]
        itemDict["Meeting House Address"] = header_data[1]
        itemDict[header_data[2].split(': ')[0]] = header_data[2].split(': ')[1]

        for pdf_page in pdf.pages:
            single_page_text = pdf_page.extract_text()
            if single_page_text is not None:
                all_text += '\n' + single_page_text
        startLoc = all_text.find('1.1')
        endLoc = all_text.find('1.19')
        trimText = all_text[startLoc:endLoc]
        splitText = re.split(r"[1]\.[0-9]+", trimText)
        for item in splitText[1:]:
            splitItem = item.strip().split(': ')
            itemDict[splitItem[0]] = splitItem[1].strip()
    except:
        pass
    return itemDict



# quakerheritage/cleanseData.py

"""Runs Pandas functionality to merge text data into workable DataFrame and creates consistent data quality.

This module allows the user to take the raw data from the Quaker Heritage Project's pdfs and enhances usefulness of data points and types.

This module contains the following functions:

- `createDataframe(url)` - Takes a list of dictionaries with varied values and keys and unifies them into a single Pandas DataFrame.
- `hygieneDataFrame(df)` - Takes the varied data in the DataFrame and applies a regular schema to it for data quality purposes.
"""

import pandas as pd
import numpy as np

# collate all data
def createDataFrame(dictList: list) -> pd.DataFrame:
    """Concatenates dictionary values and transforms them into a Pandas DataFrame.

    Args:
        dictList (array): A list object of dictionaries created from pdfs.

    Return:
        df (Pandas DataFrame): A raw and unhygiened Pandas DataFrame.
    """
    data = []
    data.append([i for i in dictList[0].keys()])
    for dct in dictList:
        data.append([i for i in dct.values()])
    df = pd.DataFrame(data, columns = data[0])
    df = df.rename(columns=df.iloc[0]).drop(df.index[0]).reset_index(drop=True)
    return df

def hygieneDataFrame(df: pd.DataFrame) -> pd.DataFrame:
    """Data quality function that iterates through selected columns within the DataFrame and corrects variant values.

    Example:
        'Date of Visit': 
            Old Value: 5th July 2015
            Hygiened Value: 2015-07-05
        'Date':
            Old Value: c1864-70, 1965; 2005
            Hygiened Value: 1864

    Args:
        df (Pandas DataFrame): A raw and unhygiened DataFrame.

    Return:
        df (Pandas DataFrame): The same DataFrame with all formatting applied for data quality.
    """
    df.rename(columns={'Historic Environment Scotland': 'Historic locality', 'Reference': 'Listing Reference'}, inplace = True)
    try:
        df['Listed status'] = df['Listed status'].apply(lambda x: 'Not listed' if 'No' in x else x)
        df['Listing Reference'] = df['Listing Reference'].apply(lambda x: 'N/A' if ('Not applicable' in x or x.upper() == 'N/A') else x)
        df['Heritage at Risk'] = np.where(df['Listed status'].str.upper() == 'NOT LISTED', 'No', df['Heritage at Risk'])
        df['Date'] = df['Date'].str.extract(r'^(\d{4})', expand=False)
        df['Architect'] = df['Architect'].replace({'Not established': 'Unknown', 'not established': 'Unknown', 'None': 'Unknown',',':';'}, regex = True)
        df['Name of contact made on site'] = df['Name of contact made on site'].replace(' and ', ';')
        df['Date of visit'] = pd.to_datetime(df['Date of visit']).apply(lambda x: x.date())
        df['Associated buildings and sites'] = df['Associated buildings and sites'].apply(lambda x: 'N/A' if ('Not applicable' in x or 'None' in x) else x)
    except:
        pass
    
    return df

def saveToCSV(df: pd.DataFrame, filepath: str) -> None:
    """Small function to save CSV with appropriate format.
    
    Args:
        df (Pandas DataFrame): collection of data for Pandas to convert to CSV.
        filepath (string): desired location for file creation.
    """
    df.to_csv(filepath, encoding='utf-8-sig')



# quakerheritage/callFunctions.py

"""Unifies the functional modules and runs the main code to create a formatted Pandas DataFrame from pdfs held by Britain Yearly Meeting.

This module runs automatically when opened, in concert with the other modules in this package. 

This module contains the following functions:

- `getOnlineData(url)` - Collects pdfs from webpage, extracts text to dictionary, creates DataFrame from all dicitonaries and hygienes data.

"""

import tkinter as tk
from tkinter import filedialog

import pandas as pd

import getWebData as gwd
import cleanseData as cd

url = "https://heritage.quaker.org.uk/"

def getOnlineData(url: str) -> pd.DataFrame: 
    """Collect online data and merge it into a Pandas DataFrame
    
    Args:
        url (string): A fixed URL for the Quaker Meeting House Heritage Project's pdf storage.
        
    Returns:
        df (Pandas DataFrame): A transformed and hygeined DataFrame.
    """
    pdfList = gwd.getUrls(url)
    dictList = []
    for pdf in pdfList[1:3]: 
        dictList.append(gwd.pdfDataExtract(pdf))
    df = cd.createDataFrame(dictList)
    df = cd.hygieneDataFrame(df)
    return df
    

if __name__ == '__main__':
    root = tk.Tk()
    root.withdraw()

    file_path = filedialog.askdirectory() + '\quakerHeritageDB.csv'
    db = getOnlineData(url)
    cd.saveToCSV(db, file_path)


import pathlib
from setuptools import setup
HERE = pathlib.Path(__file__).parent
README = (HERE / "README.md").read_text()
setup(
  name="quakerheritage",
  version="0.0.3",
  description="",
  long_description=README,
  long_description_content_type="text/markdown",
  author="",
  author_email="",
  license="AGPL",
  packages=["src/quakerheritage"],
  zip_safe=False
)